<? 
/***************************************
** Title........: HTML parse library - core functions
** Filename.....: main.inc.php
** Author.......: Elena Alekseeva aka Felenka
** Version......: 1.3
** Notes........:
** Last changed.: 15 Sept 2002
** Last change..: Parsing algorithm is optimized and restructured; comments are provided
***************************************/

require_once(INCLUDE_DIR.'const.inc.php');

/******************* basic parsing functions **************************/
// string parsing 
function parseStr($str = '', $locals = '', $globname = DEFAULT_PARSING_VARNAME){
	if (!$str) return '';
	
	$parsevar = &$GLOBALS[$globname];
	
	toArray($locals);

	$str = preg_replace(	'/('.$parsevar->varstart.')('.$parsevar->varname.')('.$parsevar->varend.')/e', 
				'(isset($locals["\\'.$parsevar->varnum.'"]) 
					? $locals["\\'.$parsevar->varnum.'"] 
					: ( isset($GLOBALS["\\'.$parsevar->varnum.'"]) 
						? $GLOBALS["\\'.$parsevar->varnum.'"] 
						: (defined("\\'.$parsevar->varnum.'") 
							? constant("\\'.$parsevar->varnum.'") 
							: ""))) ', 
				$str);
	return $str;
}

// file content parsing 
function parseFile($fname = '', $locals = '', $globname = DEFAULT_PARSING_VARNAME)
{
	if ( empty($fname) ) 
		return '';
	if ( !file_exists($fname) ) 
		return ( empty($GLOBALS[$fname]) ? '' : $GLOBALS[$fname] );

	$func = ( empty($locals['localfunc']) ? 'parseStr' : $locals['func'] );
	return $func( join('', file($fname)), $locals, $globname );
}

// string containing tags parsing 
function parseTag($str = '', $locals = '', $globname = DEFAULT_PARSING_VARNAME)
{
	if ( !$str )	return $str;
	$parsevar = &$GLOBALS[$globname];

	return preg_replace (
		'/<'.$parsevar->tagname.'([^>]*?)>('.$parsevar->taginner.')<\/'.$parsevar->tagname.'>/se', 
		'parseTagInner("\\2", "\\1", $locals, $globname)', 
		$str);	
}

// internal function, used to provide valid tag inner splitting
function getTagInner(&$str, $tagname = DEFAULT_TAGNAME)
{
	if ( !trim($str) ) // if string is empty
		return $str;
		
	$ltag = strtolower($tagname); 	// lowercased tag name
	$lstr = strtolower($str);	// lowercased $str
	
	// first occurence next starting tag in $str
	$pos1 = strpos($lstr, '<'.$ltag); 
	

	/*
		the simplest case: $str doesn't contain 
		any opening <callback ...> tags;
		it means, that tag 
		<callback ...>no other callback-tags here</callback> 
		is given, and in this case $str == tag inner
	*/
	if ( $pos1 === false )
	{	
		return array('str1' => $str, 'middle' => '', 'str2' => '');
	}
	
	// length of closing tag
	$taglen = strlen('</'.$ltag.'>'); 
	
	// add closing tag to lowercased $str
	$lstr .= '</'.$ltag.'>';
			
	// the last occurence of closing tag in $str
	$rpos2 = strrpos($lstr, '</'.$ltag.'>');
	
	/* 
		if closing tag is not found, we have unclosed callback-tag, 
		since it means, that string like 
		<callback ...>
			<callback ...>
		</callback>
		is parsed
	*/
	if ( $rpos2 == false ){
		die(	' Unclosed nested tag for '.
			$pos1.' position in string '.
			htmlentities($str) );
	}
	
	// the first occurence of closing tag in $str
	$pos2 = strpos($lstr, '</'.$ltag.'>');

	/* 
		skip all nested tags in $str (if any)
		and parse the string $str till 
		(1) opening tag is found in $str AND
		(2) closing tag is found in $str AND 
		(3) last found closing tag follows after last found opening tag AND
		(4) $str length is not exceeded
	*/
	while ( $pos1 !== false &&  		// (1)
		$pos2 !== false &&  		// (2)
		$pos2 > $pos1 && 		// (3)
		$pos2 + $taglen < $rpos2 )	// (4)
	{
		// find the next occurence of opening tag
		$pos1 = strpos($lstr, '<'.$ltag, $pos1+$taglen);

		// find the next occurence of closing tag
		$pos2 = strpos($lstr, '</'.$ltag.'>', $pos2+$taglen+1);
	}

	// part of string before the closing tag occurence
	$str1 = substr($str, 0, $pos2);
	
	// $str2 = all the rest $str after the closing tag occurence
	$str2 = substr($str, $pos2 + $taglen);
	
	/*
		if closing tag is at the end of $str, it means, 
		that parsed string looks like
		
		<callback ...>
			$str1 
		</callback>
		
		where $str1 also may contain nested callback-tags
	*/
	if ( empty($str2) )
		return array( 'str1' => $str1, 'middle' => '', 'str2' => '' );

	/* 
		if closing tag is NOT at the end of $str, 
		continue the parsing 
	*/

	// find the first occurence of the opening tag in $str2
	$pos1 = strpos(strtolower($str2), '<'.$ltag);
		
	/*
		if no opening tags found in $str2
		it means, that parsed string looks like

		<callback ...>
			$str1 
		</callback>
		$str2
		
		where $str2 contains only constant information
	*/
	if ( $pos1 === false ) 	{
		return array('str1' => $str1, 'middle' => $str2, 'str2' => '');
	}

	/*
		if opening tag is found in $str2
		it means, that parsed string looks like

		<callback ...>
			$str1 
		</callback>
		$str2
		
		where $str2 looks like 
		
		constant part of string
		<callback ...>
			tag inner
		</callback>

	*/
	
	// this is constant part of the string between two callback-tags
	$middle = substr($str2, 0, $pos1);
	
	// make valid callback-tag in $str2 
	$str2 = substr($str2, $pos1).'</'.$ltag.'>';
	
	return array('str1' => $str1, 'middle' => $middle, 'str2' => $str2);
} 

// internal function, used to parse given tag inner
function parseTagInner(	$str = '', $pars = '', $locals = '', $globname = DEFAULT_PARSING_VARNAME)
{
	$parsevar = &$GLOBALS[$globname];
	extract( getTagInner($str, $parsevar->tagorigin) );

	if ( !is_array($pars) )
		$pars = AttributesToArray($pars);
	extract($pars);

	/*
		if function name is not given or function is invalid, 
		just extract the tag inner and leave it untouched
	*/
	if ( empty($name) || (!function_exists($name)) ) {
		return 	$str1.$middle.($str2 ? parseTag($str2, $locals, $globname) : '');
	}
	
/*
	if function is basic handler, 
	it means that one of the following tags is parsed: 
	
	<callback name=parseStr  [ save=varname ]>
	<callback name=parseTag  [ save=varname ]>
	<callback name=parseFile [ save=varname ]>
	
	where [ save=varname ] means that save attribute is optional
*/

	if ( in_array($name, $parsevar->basic) ){
		// apply basic handler to tag inner
		$res = 	$name($str1, $locals, $globname);

		// save function result in global variable, if its name is given
		if ( !empty($save) )
			$GLOBALS[ $save ] = $res;
			
		return 	$res.$middle.($str2 ? parseTag($str2, $locals, $globname) : '');
	}	

/*
	 if function is advanced handler,
	 it means that one of the following tags is parsed: 
	 
<callback name=fileInclude [ save=varname ] [ func=funcname ] [ ffunc=funcname ]> 
<callback name=makeBool    var=varname delim=delimiter [ type=invert ] [ save=varname ] [ func=funcname ]> 
<callback name=makeRepeat  arr=varname [ save=varname ] [ func=funcname ]> 
<callback name=makeSwitch  arr=varname delim=delimiter idx=value [ save=varname ] [ func=funcname ] [ ffunc=funcname ]> 

	where [...] - means optional attributes
	
NOTE: 'makeSwitch' handler is only suggested to be added
*/

	if ( in_array($name, $parsevar->stand) ){
		// apply advanced handler to tag inner
		$res = $name($str1, $pars, $locals, $globname);

		// save function result in global variable, if its name is given
		if ( !empty($save) )
			$GLOBALS[ $save ] = $res;
		
		return 	$res.$middle.( $str2 ? parseTag($str2, $locals, $globname) : '' );
	}	

/*
	If we're here, it means that the following tag is parsed: 
	
	<callback name=funcname [ save=varname ]>
	
	where [ save=varname ] means that save attribute is optional

*/	
	$res = $name($str1);
	
	if ( !empty($save) )
		$GLOBALS[ $save ] = $res;
		
	return 	$res.$middle.( $str2 ? parseTag($str2, $locals, $globname) : '' );
}

/******************* another helpful functions ************************/
function parseStrTag($str = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{  return parseStr( parseTag($str, $v, $globname), $v, $globname); }

function parseTagStr($str = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{  return parseTag( parseStr($str, $v, $globname), $v, $globname); }

function parseFileTag($str = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{  return parseFile( parseTag($str, $v, $globname), $v, $globname);  }

function parseTagFile($fname = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{  return parseTag( parseFile($fname, $v, $globname), $v, $globname);  }

function parseStrFile($fname = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{  return parseStr( parseFile($fname, $v, $globname), $v, $globname);  }

function parseStrTagFile($fname = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{ return parseStr(parseTag(parseFile($fname, $v, $globname), $v, $globname), $v, $globname); }

function parseTagStrFile($fname = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{ return parseTag(parseStr(parseFile($fname, $v, $globname), $v, $globname), $v, $globname); }

/************** rare used helpful functions - just added for completeness **************/
function parseFileStr($str = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{  return parseFile( parseStr($str, $v, $globname), $v, $globname);  }

function parseFileTagStr($str = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{ return parseFile(parseTag(parseStr($str, $v, $globname), $v, $globname), $v, $globname); }

function parseFileStrTag($str = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{  return parseFile(parseStr(parseTag($str, $v, $globname), $v, $globname), $v, $globname); }

function parseStrFileTag($str = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{  return parseStr(parseFile(parseTag($str, $v, $globname), $v, $globname), $v, $globname); }

function parseTagFileStr($str = '', $v = '', $globname = DEFAULT_PARSING_VARNAME)
{ return parseTag(parseFile(parseStr($str, $v, $globname), $v, $globname), $v, $globname); }
?>